/*
 * listcap.c
 *
 * capture history (?@...) sample.
 */
#include <stdio.h>
#include "oniguruma.h"

extern int main(int argc, char* argv[])
{
  int r;
  unsigned char *start, *range, *end;
  regex_t* reg;
  RegErrorInfo einfo;
  RegRegion *region;
  RegSyntaxType syntax;

  static unsigned char* pattern = "(a)(?@.b.)+(DEF)";
  static unsigned char* str = "aabcwbdjbqpbsibvbbbbbbabcbbcpbvbbdbbbbbbDEF";

  regex_copy_syntax(&syntax, REG_SYNTAX_DEFAULT);
  syntax.op2 |= REG_SYN_OP2_ATMARK_CAPTURE_HISTORY; /* enable capture hostory */
  r = regex_new(&reg, pattern, pattern + strlen(pattern),
	REG_OPTION_DEFAULT, REGCODE_ASCII, &syntax, &einfo);
  if (r != REG_NORMAL) {
    char s[REG_MAX_ERROR_MESSAGE_LEN];
    regex_error_code_to_str(s, r, &einfo);
    fprintf(stderr, "ERROR: %s\n", s);
    exit(-1);
  }

  region = regex_region_new();

  end   = str + strlen(str);
  start = str;
  range = end;
  r = regex_search(reg, str, end, start, range, region, REG_OPTION_NONE);
  if (r >= 0) {
    int i, j;

    fprintf(stderr, "match at %d\n", r);
    for (i = 0; i < region->num_regs; i++) {
      fprintf(stderr, "%d: (%d-%d)\n", i, region->beg[i], region->end[i]);
    }
    fprintf(stderr, "\n");

    /* capture history */
    for (i = 1; i <= region->num_regs; i++) {
      if (REG_IS_CAPTURE_HISTORY_GROUP(region, i)) {
	RegRegion* caps = region->list[i];
	fprintf(stderr, "%d: %d\n", i, caps->num_regs);
	for (j = 0; j < caps->num_regs; j++) {
	  fprintf(stderr, "  (%d-%d)\n", caps->beg[j], caps->end[j]);
	}
      }
    }
  }
  else if (r == REG_MISMATCH) {
    fprintf(stderr, "search fail\n");
  }
  else { /* error */
    char s[REG_MAX_ERROR_MESSAGE_LEN];
    regex_error_code_to_str(s, r);
    exit(-1);
  }

  regex_region_free(region, 1 /* 1:free self, 0:free contents only */);
  regex_free(reg);
  regex_end();
  return 0;
}
